﻿Type=StaticCode
Version=2.22
@EndOfDesignText@
'Code module
 
Sub Process_Globals
	'VERSION: 1.03
	Type GraphInternal (originX As Int, originY As Int, maxY As Int, intervalX As Float, gw As Int, gh As Int)
	Type Graph (GI As GraphInternal, Title As String, YAxis As String, XAxis As String, YStart As Float, _ 
	YEnd As Float, YInterval As Float, AxisColor As Int)
	Type LinePoint (X As String, Y As Float, YArray() As Float, ShowTick As Boolean)
	Type LineData (Points As List, LinesColors As List, Target As Panel, Canvas As Canvas)
	Type BarData (Points As List, BarsColors As List, Target As Panel, Canvas As Canvas, Stacked As Boolean, BarsWidth As Int)
End Sub
#Region Bar chart methods
 
Sub AddBarPoint (BD As BarData, X As String, YArray() As Float)
	If BD.Points.IsInitialized = False Then 
		BD.Points.Initialize
		'Add a "dummy" point as the first point to avoid drawing on the Y axis.
		Dim b As LinePoint
		b.Initialize
		b.X = ""
		b.ShowTick = False
		BD.Points.Add(b)
	End If
	Dim b As LinePoint 'using the same structure of Line charts
	b.Initialize
	b.X = X
	b.YArray = YArray
	b.ShowTick = True
	BD.Points.Add(b)
End Sub
 
Sub AddBarColor(BD As BarData, Color As Int)
	If BD.BarsColors.IsInitialized = False Then BD.BarsColors.Initialize
	BD.BarsColors.Add(Color)
End Sub
 
Sub DrawBarsChart(G As Graph, BD As BarData, BackColor As Int)
	If BD.Points.Size = 0 Then
		ToastMessageShow("Missing bars points.", True)
		Return
	End If
	BD.Canvas.Initialize(BD.Target)
	BD.Canvas.DrawColor(BackColor)
	Dim point As LinePoint
	point = BD.Points.Get(1)
	'In stacked mode we don't need to leave space for all the bars.
	drawGraph(G, BD.Canvas, BD.Target, BD.Points, True, BD.BarsWidth / point.YArray.Length)
	'Draw bars
	Dim Rect As Rect
	Rect.Initialize(0, 0, 0, G.GI.originY)
	Dim numberOfBars As Int
	numberOfBars = point.YArray.Length
	'draw all bars
	For i = BD.Points.Size - 1 To 1 Step - 1 
		point = BD.Points.Get(i)
		For a = 0 To numberOfBars - 1			
			Rect.Left = G.GI.originX + G.GI.intervalX * i - BD.BarsWidth / 2
			Rect.Right = Rect.Left + BD.BarsWidth
			If a = 0 Then
				Rect.Top = calcPointToPixel(0, G)
			Else
				Rect.Left = Rect.Left + BD.BarsWidth / 4
			End If
			Rect.Bottom = Rect.Top
			Rect.Top = Rect.Bottom + calcPointToPixel(point.YArray(a), G) - G.GI.originY
			BD.Canvas.DrawRect(Rect, BD.BarsColors.Get(a), True, 1dip)
		Next
	Next
	BD.Target.Invalidate
End Sub
'Draws the graph layout (not including the data)
'This method is used by Bars and Lines charts
 
Sub drawGraph (G As Graph, Canvas As Canvas, Target As View, Points As List, Bars As Boolean, BarsWidth As Int)
	Dim GI As GraphInternal
	G.GI = GI
	GI.Initialize
	GI.gw = Target.Width - 70dip 'graph width
	GI.gh = Target.Height - 100dip 'graph height
	GI.originX = 50dip
	GI.originY = Target.Height - 60dip
	GI.maxY = 40dip
	If Bars Then
		Dim p As LinePoint
		p = Points.Get(1)
		GI.intervalX = (GI.gw - p.YArray.Length / 2 * BarsWidth) / (Points.Size - 1)
	Else
		GI.intervalX = GI.gw / (Points.Size - 1)
	End If
	'Draw Y axis
	Canvas.DrawLine(GI.originX, GI.originY + 2dip, GI.originX, GI.maxY - 2dip, G.AxisColor, 2dip)
	For i = 0 To (G.YEnd - G.YStart) / G.Yinterval + 1
		Dim y As Int
		Dim yValue As Float
		yValue = G.YStart + G.YInterval * i
		If yValue > G.YEnd Then Continue
		y = calcPointToPixel(yValue, G)
		Canvas.DrawLine(GI.originX, y, GI.originX - 5dip, y, G.AxisColor, 2dip)
		Canvas.DrawLine(GI.originX, y, Target.Width, y, G.AxisColor, 1dip)
		Canvas.DrawText(NumberFormat(yValue, 1, 2), GI.originX - 8dip, y + 5dip, Typeface.DEFAULT, 12, G.AxisColor, "RIGHT")
	Next
	'Draw titles
	'	Canvas.DrawText(G.Title, Target.Width / 2, 30dip, Typeface.DEFAULT_BOLD, 15, G.AxisColor, "CENTER")
	Canvas.DrawText(G.XAxis, Target.Width / 2, GI.originY + 45dip, Typeface.DEFAULT, 14, G.AxisColor, "CENTER")
	Canvas.DrawTextRotated(G.YAxis, 15dip, Target.Height / 2, Typeface.DEFAULT, 14, G.AxisColor, "CENTER", - 90)
	'Draw X axis
	Canvas.DrawLine(GI.originX, GI.originY, Target.Width, GI.originY, G.AxisColor, 2dip)
	'For i = 0 To Points.Size - 1
	For i = Points.Size - 1 To 0 Step - 1
		Dim p As LinePoint
		p = Points.Get(i)
		If p.ShowTick Then
			Dim x As Int
			x = GI.originX + i * GI.intervalX
			Canvas.DrawLine(x, GI.originY, x, GI.originY + 5dip, G.AxisColor, 2dip)
			If Bars = False Then Canvas.DrawLine(x, GI.originY, x, GI.maxY, G.AxisColor, 1dip) 'vertical lines
			If p.x.Length > 0 Then
				Canvas.DrawTextRotated(p.x, x, GI.originY + 12dip, Typeface.DEFAULT, 12, G.AxisColor, "RIGHT", - 90)
			End If
		End If
	Next
End Sub
 
Sub calcPointToPixel(py As Float, G As Graph) As Int
	Return G.GI.originY - (G.GI.originY - G.GI.maxY) * (py - G.YStart) / (G.YEnd - G.YStart)
End Sub
#End Region